/*
 * This program is free software: you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or (at your
 * option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
 * more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <https://www.gnu.org/licenses/>.
 */
package com.inteligr8.alfresco.acs;

import java.util.List;
import java.util.UUID;

import jakarta.ws.rs.NotFoundException;

import org.junit.jupiter.api.Assertions;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.condition.EnabledIf;

import com.inteligr8.alfresco.acs.api.DiscoveryApi;
import com.inteligr8.alfresco.acs.api.NodesApi;
import com.inteligr8.alfresco.acs.api.V0Api;
import com.inteligr8.alfresco.acs.model.RepositoryInfo;
import com.inteligr8.alfresco.acs.model.v0.ClassInfo;
import com.inteligr8.alfresco.acs.model.v0.MimeTypesData;
import com.inteligr8.alfresco.acs.model.v0.PropertyInfo;
import com.inteligr8.alfresco.acs.model.v0.ServerData;

public abstract class ConnectionClientIT extends ConditionalIT {
	
	public abstract AcsPublicRestApi getApi();
	
	@Test
	@EnabledIf("hostExists")
	public void testDiscovery() {
		DiscoveryApi api = this.getApi().getDiscoveryApi();
		RepositoryInfo repoInfo = api.getRepositoryInformation().getEntry().getRepository();
		
		Assertions.assertNotNull(repoInfo);
		Assertions.assertFalse(repoInfo.getStatus().isIsReadOnly());
		Assertions.assertEquals("6", repoInfo.getVersion().getMajor());
	}
	
	@Test
	@EnabledIf("hostExists")
	public void testMissingNode() {
		NodesApi api = this.getApi().getNodesApi();
		
		Assertions.assertThrows(NotFoundException.class, () -> {
			api.getNode(UUID.randomUUID().toString(), null, null, null);
		});
	}
	
	@Test
	@EnabledIf("hostExists")
	public void testV0ServerInfo() {
		V0Api api = this.getApi().getV0Api();
		ServerData server = api.getServer();
		
		Assertions.assertNotNull(server);
		Assertions.assertNotNull(server.getData());
		Assertions.assertNotNull(server.getData().getEdition());
		Assertions.assertTrue(server.getData().getEdition().length() > 5);
		Assertions.assertTrue(server.getData().getVersion().length() > 5);
		Assertions.assertTrue(server.getData().getSchema().length() > 3);
	}
	
	@Test
	@EnabledIf("hostExists")
	public void testV0MimeTypes() {
		V0Api api = this.getApi().getV0Api();
		MimeTypesData mimeTypes = api.getMimeTypes();
		
		Assertions.assertNotNull(mimeTypes);
		Assertions.assertNotNull(mimeTypes.getData());
		Assertions.assertTrue(mimeTypes.getData().getMimeTypes().size() > 50);
		Assertions.assertNotNull(mimeTypes.getData().getMimeTypeInfo("application/pdf"));
		Assertions.assertEquals("pdf", mimeTypes.getData().getMimeTypeInfo("application/pdf").getExtensions().getDefault());
	}
	
	@Test
	@EnabledIf("hostExists")
	public void testV0Classes() {
		V0Api api = this.getApi().getV0Api();
		List<ClassInfo> classes = api.getClasses(null, null, null);
		
		Assertions.assertNotNull(classes);
		Assertions.assertTrue(classes.size() > 50);
	}
	
	@Test
	@EnabledIf("hostExists")
	public void testV0Property() {
		V0Api api = this.getApi().getV0Api();
		List<PropertyInfo> props = api.getProperties(null, "cm:content", null);
		
		Assertions.assertNotNull(props);
		Assertions.assertEquals(1, props.size());
		Assertions.assertEquals("d:content", props.iterator().next().getDataType());
	}
	
}
